@@ -21,13 +21,14 @@
 #include <config.h>
 #endif
 
-#include "screenshooter-utils.h"
+#include "libscreenshooter.h"
 
 
 
 /* Set default values for cli args */
 gboolean version = FALSE;
 gboolean window = FALSE;
+gboolean fullscreen = FALSE;
 gboolean no_save_dialog = FALSE;
 gboolean preferences = FALSE;
 gchar *screenshot_dir;
@@ -49,6 +50,10 @@ static GOptionEntry entries[] =
         N_("Take a screenshot of the active window"),
         NULL
     },
+    {   "fullscreen", 'f', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &fullscreen,
+        N_("Take a screenshot of the desktop"),
+        NULL
+    },
     {		"delay", 'd', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_INT, &delay,
        N_("Delay in seconds before taking the screenshot"),
        NULL
@@ -76,31 +81,16 @@ int main(int argc, char **argv)
   GError *cli_error = NULL;
   GdkPixbuf *screenshot;
   ScreenshotData *sd = g_new0 (ScreenshotData, 1);
-  XfceRc *rc;
   gchar *rc_file;
   
+  xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
+  
   /* Get the path to the conf file */
   rc_file = g_build_filename (xfce_get_homedir(), ".config", "xfce4", 
                               "xfce4-screenshooter", NULL);
   
-  /* If the file exists, we parse it to get the default save folder. Else we use
-  the home dir. */
-  if (g_file_test (rc_file, G_FILE_TEST_EXISTS))
-    {
-      rc = xfce_rc_simple_open (rc_file, TRUE);
-      screenshot_dir = g_strdup (xfce_rc_read_entry (rc, "screenshot_dir", 
-                               DEFAULT_SAVE_DIRECTORY));
-      sd->screenshot_dir = screenshot_dir;
-      xfce_rc_close (rc);
-    }
-  else
-    {
-      screenshot_dir = g_strdup (DEFAULT_SAVE_DIRECTORY);
-      sd->screenshot_dir = screenshot_dir;
-    }
-
-  xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
-  
+  screenshooter_read_rc_file (rc_file, sd, !(window || fullscreen));
+    
   /* Print a message to advise to use help when a non existing cli option is
   passed to the executable. */  
   if (!gtk_init_with_args(&argc, &argv, _(""), entries, PACKAGE, &cli_error))
@@ -125,11 +115,15 @@ int main(int argc, char **argv)
   the desktop.*/
   if (window)
     {
-      sd->whole_screen = 0;    
+      sd->mode = ACTIVE_WINDOW;    
+    }
+  else if (fullscreen)
+    {
+      sd->mode = FULLSCREEN;
     }
   else
     {
-      sd->whole_screen = 1;
+      sd->mode = FULLSCREEN;
     }
   
   /* Wether to show the save dialog allowing to choose a filename and a save 
@@ -143,76 +137,98 @@ int main(int argc, char **argv)
       sd->show_save_dialog = 1;
     }
 
-  sd->screenshot_delay = delay;
+  sd->delay = delay;
   
-  /* Verify that the user gave a valid directory name */
-  if (g_file_test (screenshot_dir, G_FILE_TEST_IS_DIR))
+  /* If the user gave a directory name, verify that it is valid */
+  if (screenshot_dir != NULL)  
     {
-    /* Check if the path is absolute, if not make it absolute */
-      if (g_path_is_absolute (screenshot_dir))
-        { 
-          sd->screenshot_dir = screenshot_dir;
+      if (g_file_test (screenshot_dir, G_FILE_TEST_IS_DIR))
+        {
+          /* Check if the path is absolute, if not make it absolute */
+          if (g_path_is_absolute (screenshot_dir))
+            { 
+              g_free (sd->screenshot_dir);
+              sd->screenshot_dir = screenshot_dir;              
+            }
+          else
+            {
+              g_free (sd->screenshot_dir);
+              sd->screenshot_dir = 
+              g_build_filename (g_get_current_dir (), screenshot_dir, NULL);
+              g_free (screenshot_dir);
+            }
+        }
+      else
+        {
+          g_warning ("%s is not a valid directory, the default directory will be used.", 
+                     screenshot_dir);
+          g_free (screenshot_dir);
         }
-    else
-      {
-        screenshot_dir = 
-          g_build_filename (g_get_current_dir (), screenshot_dir, NULL);
-        sd->screenshot_dir = screenshot_dir;
-      }
-    }
-  else
-    {
-      g_warning ("%s is not a valid directory, the default directory will be used.", 
-                 screenshot_dir);
     }
   
-  /* If -p is given, show the preferences dialog, else just take the screenshots
-  with the given options */
-  if (!preferences)
+  /* If a mode cli option is given, take the screenshot accordingly. */
+  if (fullscreen || window)
     {
-      screenshot = take_screenshot (sd);
-      save_screenshot (screenshot, sd);
+      screenshot = screenshooter_take_screenshot (sd->mode, sd->delay);
+      screenshooter_save_screenshot (screenshot, sd->show_save_dialog, 
+                                     sd->screenshot_dir);
     
       g_object_unref (screenshot);
     }
+  /* If -p is given, show the preferences dialog */
+  else if (preferences)
+    {
+      screenshooter_preferences_dialog (rc_file, sd->screenshot_dir);
+    }
+  /* Else we just show up the main application */
   else
     {
-      GtkWidget * chooser;
-      gint dialog_response;
-      gchar * dir;
+      GtkWidget *dialog;
+      gint response;
       
-      /* The preferences dialog is a plain gtk_file_chooser, we just get the
-      folder the user selected and write it in the conf file*/
-      dir = screenshot_dir;
-
-      chooser = 
-        gtk_file_chooser_dialog_new (_("Default save folder"),
-                                     NULL,
-                                     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-                                     NULL);
-      gtk_dialog_set_default_response (GTK_DIALOG (chooser), GTK_RESPONSE_ACCEPT);
-      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), dir);
-
-      dialog_response = gtk_dialog_run(GTK_DIALOG (chooser));
-
-      if (dialog_response == GTK_RESPONSE_ACCEPT)
-	      {
-	        dir = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+      /* Read the preferences */
+      screenshooter_read_rc_file (rc_file, sd, FALSE);
       
-          rc = xfce_rc_simple_open (rc_file, FALSE);
-          xfce_rc_write_entry (rc, "screenshot_dir", dir);
-          xfce_rc_close (rc);
+      /* Set the dialog up */
+      dialog = screenshooter_dialog_new (sd, FALSE);
       
-          g_free (dir);
-	      }
-      gtk_widget_destroy (GTK_WIDGET (chooser));
+      /* Run the dialog and destroy it, so that it's not grabbed in active
+         window mode */
+      response = gtk_dialog_run (GTK_DIALOG (dialog));
+      
+      gtk_widget_destroy (dialog);
+      
+      if (response == GTK_RESPONSE_OK)
+        {
+          gchar *screenshot_path = NULL;
+          
+          /* Take the screenshot */
+          screenshot = screenshooter_take_screenshot (sd->mode, sd->delay);
+          screenshot_path =
+            screenshooter_save_screenshot (screenshot, sd->show_save_dialog, 
+                                           sd->screenshot_dir);
+          g_object_unref (screenshot);
+          
+          /* Open the screenshot */
+          #ifdef HAVE_GIO
+          if (screenshot_path != NULL)
+            {
+              screenshooter_open_screenshot (screenshot_path, sd->app);
+              g_free (screenshot_path);
+            }
+          #endif
+          
+          /* Save preferences */     
+          screenshooter_write_rc_file (rc_file, sd);
+        }
     }
   
   g_free (sd->screenshot_dir);
+  #ifdef HAVE_GIO
+  g_free (sd->app);
+  #endif
   g_free (sd);
   g_free (rc_file);
-  
+    
   return 0;
 }
